@@ -933,6 +933,37 @@ class Sequence(Generic[T], Iterable[T]):
         ['a', 'b', 'c', 'd', 'e']
         """
         return FlatteningSequence(self, transform)
+    
+
+    def group_by(self, key_selector: Callable[[T], K]) -> Dict[K, Sequence[T]]:
+        """
+         Returns a dictionary with keys being the result of [key_selector] function being invoked on each element of original collection
+        and values being the corresponding elements of original collection.
+
+         Example 1:
+        >>> lst = [1, 2, 3, 4, 5]
+        >>> it(lst).group_by(lambda x: x%2)
+        {1: [1, 3, 5], 0: [2, 4]}
+        """
+        return self.group_by_to(dict(), key_selector)
+    
+
+    def group_by_to(self, destination: Dict[K, List[T]], key_selector: Callable[[T], K]) -> Dict[K, List[T]]:
+        """
+         Returns a dictionary with keys being the result of [key_selector] function being invoked on each element of original collection
+        and values being the corresponding elements of original collection.
+
+         Example 1:
+        >>> lst = [1, 2, 3, 4, 5]
+        >>> it(lst).group_by_to({}, lambda x: x%2)
+        {1: [1, 3, 5], 0: [2, 4]}
+        """
+        for e in self:
+            k = key_selector(e)
+            if k not in destination:
+                destination[k] = []
+            destination[k].append(e)
+        return destination
 
     def foreach(self, action: Callable[[T], None]) -> None:
         """
